function [PDF,MEAN,MEDIAN,MODE,VARIANCE,CREDIBLE_INTERVAL]=...
         posteriori(N, R, M)
%--------------------------------------------------------------------------
%   Log
%   v4.0 - PAPER VERSION
%
% Empirical Bayesian analysis of dichotomic data with errors and repeated classifications
%
%   Magda Carvalho Pires - UFMG - Brazil
%   Roberto da Costa Quinino - UFMG - Brazil
%   Emilio Suyama - UFMG - Brazil
%   Anderson Lacio Galindo Trindade - USP - Brazil
%
%   Syntax:
%
%   PDF=POSTERIORI(N,R,M)
%   %   [PDF,MEAN]=POSTERIORI(...)
%   [PDF,MEAN,MEDIAN]=POSTERIORI(...)
%   [PDF,MEAN,MEDIAN,MODE]=POSTERIORI(...)
%   [PDF,MEAN,MEDIAN,MODE,VARIANCE]=POSTERIORI(...)
%   [PDF,MEAN,MEDIAN,MODE,VARIANCE,CREDIBLE_INTERVAL]=POSTERIORI(...)
%
%   N   -    sample size
%   R   -    scalar: Number of items considered conforming. In this case,  prior distribution U(0,1) for classification errors 
%            is used.
%            vector: number of conforming classifications of each of N ispected items. Empirical distribution for classification
%            erros is used. 
%   M   -    number of repetitive classifications of each inspected item
%--------------------------------------------------------------------------

% Criando as variaveis globais e desligando mensagem de erro da funo
% NCHOOSEK
global n r m a1 b1 a2 b2 a b xint yint deltaxint mydisplayiter pteorico e1teorico e2teorico
mydisplayiter='off';
warning off MATLAB:nchoosek:LargeCoefficient

% Verificando os argumentos de entrada
if nargin < 3
    error('POSTERIORI requires at least three input arguments.');
else
    % Testando o valor de N
    if length(N)~=1
        error('N should be a scalar value.');
    elseif floor(N)<=0
        error('N should be a integer > 0');
    else
        n=floor(N);
    end
    % Testando o valor de R
    if length(R)==1
        if floor(R)<0 | floor(R)>n
            error('R should be a integer between 0 and N');
        else
            r=floor(R);
        end
        if length(M)~=1
            error('M should be a scalar value.');
        elseif floor(M)<=0
            error('M should be a integer > 0');
        else
            m=floor(M);
        end
    elseif length(R)==0
        error('N should be a scalar or a vector');
    else
        if N~=length(R)
            error('Length of vector R should be N');
        end
        if length(M)~=1
            error('M should be a scalar value.');
        elseif floor(M)<=0
            error('M should be a integer > 0');
        else
            m=floor(M);
        end
        if sum(floor(R)<0)>0 | sum(floor(R)>m)>0
            error('All elements of vector R should be between 0 and M');
        else
            r=sum(floor(R)>(0.5*m));
        end
    end
    a=1; b=1; a1=1; b1=1; a2=1; b2=1;
    conf=0.95; graf=1; npontos=200;
    if nargin>=4
        if C<0 | C>1
            error('C should be real value between 0 and 1');
        else
            conf=C;
        end
    end
    if nargin>=5
        if GRAF_FLAG==1
            graf=1;
        else
            graf=0;
        end
    end
    if nargin>=6
        if floor(NPOINTS)<50
            npontos=50;
        else
            npontos=floor(NPOINTS);
        end
    end
end

% Inicializando
me1=[]; %mdia erro 1
se1=[]; %desvio erro 1
me2=[]; %mdia erro 2
se2=[]; %desvio erro 2

% Estimando os erros e1 e e2
if m>1 & length(R)>1
    indice=find(R>(0.5*m));
    if length(indice)>1
        vmaior=R(indice);
        me1=mean((m-vmaior)/m);
        se1=std((m-vmaior)/m);
        if se1>0
            [a1,b1]=pbeta(me1,se1);
        else
           warning('Foi impossvel estimar o erro e1 com a amostra fornecida. Uma a priori no informativa ser utilizada.')
        end
    end
    
    indice=find(R<=(0.5*m));
    if length(indice)>1
        vmenor=R(indice);
        me2=mean((vmenor)/m);
        se2=std((vmenor)/m);
        if se2>0
            [a2,b2]=pbeta(me2,se2);
        else
           warning('Foi impossvel estimar o erro e2 com a amostra fornecida. Uma a priori no informativa ser utilizada.')
        end
    end
end

% Cria a tabela de probabilidades da binomial acumulada
% Trata-se de uma discretizao para evitar o clculo repetitivo
% ao longo da execuo do programa
nint=10000;
xint=linspace(0,1,nint+1);
yint=binocdf(0.5*m,m,xint);
deltaxint=1/nint;

% Calcula a funcao posteriori em "npontos" no intervalo [0;1]
% (note que o intervalo  aproximadamente [0;1])
vp=     linspace(1e-6,1-1e-6,npontos);
vpdf=   f_pwitherror(vp);

% Calcula as estatsticas: mdia, mediana, moda e variancia
% e o intervalo de credibilidade
media       =fmedia(vp,vpdf);
mediana     =fmediana(vp,vpdf);
moda        =fmoda(vp,vpdf);
variancia   =fvariancia(vp,vpdf,media);
[lils1]     =credib(vp,vpdf,[0.99*moda(1) 1.01*moda(1)],conf);

PDF=vpdf;
MEAN=media;
MEDIAN=mediana;
MODE=moda;
VARIANCE=variancia;
CREDIBLE_INTERVAL=lils1;

% Fazendo o grafico
if graf==1
    nc=4;
    texto='';
    texto=strvcat(texto, ['mean = ' num2str(media,nc)]);
    texto=strvcat(texto, ['median = ' num2str(mediana,nc)]);
    texto=strvcat(texto, ['mode = ' num2str(moda(1),nc) ' (pdf=' num2str(f_pwitherror(moda(1)),nc) ')']);
    for i=2:length(moda)
        if moda(i)>=0
            texto=strvcat(texto, ['mode = ' num2str(moda(i),nc) ' (pdf=' num2str(f_pwitherror(moda(i)),nc)  ')']);
        end
    end
    texto=strvcat(texto, ['variance = ' num2str(variancia,nc)]);    
    figure;
    plot(vp,vpdf,'r.-');
    hold on;
    tempx=lils1;
    tempy=spline(vp,vpdf,tempx);
    hold on;
    hold on;
    box on;
    texto=strvcat(texto, ['cred. interval (' num2str(conf*100,4) '%) = [' num2str(lils1(1),nc) ';' num2str(lils1(2),nc) ']']);
    str=['n = ' num2str(n) '; r= ' num2str(r) '; m= ' num2str(m)];
    title(str);
	
    xlabel('p');
    ylabel('f(p|r,n,m)');
	
    set(gcf,'Color',[1 1 1]);
    eixox=get(gca,'Xlim');
    eixoy=get(gca,'Ylim');
    text(eixox(1)+(eixox(2)-eixox(1))*0.05,eixoy(2)-(eixoy(2)-eixoy(1))*0.05,texto,'FontSize',10,'VerticalAlignment','top','EdgeColor','black');
    drawnow
    if length(R)>1
        filename=['empirical_chart_p' num2str(pteorico*100,3) '_n' num2str(n,3) '_e1' num2str(e1teorico*100,3) '_e2' num2str(e2teorico*100,3) '_m' num2str(m,3)];
    else
        filename=['noninformative_chart_p' num2str(pteorico*100,3) '_n' num2str(n,3) '_e1' num2str(e1teorico*100,3) '_e2' num2str(e2teorico*100,3) '_m' num2str(m,3)];
    end
    print('-dtiff','-r96',filename);
    %saveas(gcf,filename,'fig');
    close(gcf);
    drawnow
end
warning on MATLAB:nchoosek:LargeCoefficient
%--------------------------------------------------------------------------
% FIM DA FUNO posteriori
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function [a,b]=pbeta(x,s)
% Clculo do alfa e beta de uma distribuio a partir da mdia e do desvio
% padro
% x = mdia
% s = desvio
s2=s^2;
a=-x*(s2+x^2-x)/s2;
b=(s2+x^2-x)*(x-1)/s2;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=fmedia(x,Px)
% Calculo da mdia no intervalo [0,1] segundo a definio de mdia 
% integral. Px  um vetor de densidade de probabilidade
out=trapz(x,x.*Px);
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=fmediana(x,Px)
% Calculo da mediana no intervalo [0,1].  Px  um vetor de densidade de
% probabilidade
y=cumtrapz(x,Px);
indice=find(y>0.5);
x2=x(indice(1));
y2=y(indice(1))-0.5;
indice=find(y<=0.5);
x1=x(indice(length(indice)));
y1=y(indice(length(indice)))-0.5;
maxiter=200;
maxdif=1e-6;
i=0;
xm=(x2+x1)/2;
while abs(y2-y1)>maxdif & i<maxiter
    i=i+1;
    xm=(x2+x1)/2;
    vx=linspace(0,xm,100);
    vy=spline(x,Px,vx);
    ym=trapz(vx,vy)-0.5;
    if y1*ym<0
        y2=ym;
        x2=xm;
    else
        y1=ym;
        x1=xm;
    end
end
out=xm;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=fmoda(x,Px)
% Clculo da moda (ou de at duas modas). 
global myx myy mydisplayiter
n = 11;
M=zeros(n,3);
delta=1/(n-1);
myx=x;
myy=Px;
options=optimset('LargeScale','off','Display',mydisplayiter,'MaxFunEvals',1000,'MaxIter',1000);
cont=0;
for i=1:n
    x0=(i-1)*delta;
    [moda,fval,exitflag]=fmincon(@objfunModa,x0,[],[],[],[],[0],[1],@confunModa,options);
    cont=cont+1;
    M(i,1)=moda;
    M(i,2)=-fval;
    M(i,3)=exitflag;
end
MS=sortrows(M,2);
indice=find(abs(MS(:,1)-MS(n,1))<1e-3);
moda1=mean(MS(indice,1));
fval1=-objfunModa(moda1);
[lin,col]=size(indice);
if lin==n
    moda2=-1;
else
    MS(indice,:)=[];
    [lin,col]=size(MS);
    indice=find(abs(MS(:,1)-MS(lin,1))<1e-3);
    moda2=mean(MS(indice,1));
    fval2=-objfunModa(moda2);
    if abs(fval2-fval1)>1e-3 | abs(moda2-moda1)<1e-3
        moda2=-1;
    end
end
out=zeros(1,2);
out(1,1)=moda1;
out(1,2)=moda2;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=objfunModa(x)
% Funo objetivo para determinao da Moda. O objetivo  minimizar o
% negativo do vetor de densidade de probabidade (em outras palavras,
% maximizar o vetor de probabilidade de densidade)
global myx myy
out=-spline(myx,myy,x);
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function [c,ceq]=confunModa(x)
% Trata-se das restries aplicadas ao processo de mininizao para a 
% determinao da moda. Na verdade,  uma funo de restrio DUMMY, j que
% o mtodo de otimizao que eu uso exige que se coloque uma funo de
% restrio
% Restrio (inequacao)
c=[];
% Restricao (equacao)
ceq=0;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=fvariancia(x,Px,mu)
% Determinao da variancia utilizando a definio integral
y=Px.*(x-mu).^2;
out=trapz(x,y);
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function [lils]=credib(x,Px,x0,conf)
% Determinao do intervalo de credibilidade, que trata-se de um processo
% de otimizao: Qual o menor intervalo em x que contm nele uma dada
% densidade de probabilidade acumulada, especificada em conf?
global myx myy mydelta myconf mydisplayiter
npasso=1000;
myx=x;
myy=Px;
myconf=conf;
minimo=min(x);
maximo=max(x);
if x0(1)<minimo
    x0(1)=minimo;
end
if x0(2)>maximo
    x0(2)=maximo;
end

options=optimset('LargeScale','off','Display',mydisplayiter,'MaxFunEvals',...
                 1000,'MaxIter',1000);
[lils,fval,exitflag]=fmincon(@objfunCred,x0,[],[],[],[],[minimo minimo],...
                              [maximo maximo],@confunCred,options);

cont=1;
while exitflag<=0 & cont<10
    clc;
    disp('*********************************************************************');
    disp('Falha na determinao do intervalo de credibilidade...               ');
    disp('Tentando novo chute inicial...                                       ');
    disp(['Iteracao atual: ' num2str(cont) '                                     ']);
    disp('*********************************************************************');    
    cont=cont+1;
    [temp,i]=max(myy);
    xmaximo=myx(i);
    x0=[xmaximo-rand*(xmaximo-minimo) xmaximo+rand*(maximo-xmaximo)];
    [lils,fval,exitflag]=fmincon(@objfunCred,x0,[],[],[],[],[minimo minimo],...
                                 [maximo maximo],@confunCred,options);
end
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=objfunCred(x)
% Funo objetivo do intervalo de credibilidade: minimizar a diferena
% entre o limite inferior e superior
out=(x(2)-x(1))^2;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function [c,ceq]=confunCred(x)
% Restrio ao processo de otimizao na determinao do intervalo de
% credibilidade: a diferena entre o limite inferior e superior deve ser
% mnima sujeita  restrio de que a rea entre estes dois limites deve
% ser igual  conf (nvel de confian desejado)
global myxx myyy mydelta myconf
% Restrio (inequacao)
c=[];
% Restricao (equacao)
ceq=mycdf(x(2))-mycdf(x(1))-myconf;
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
%                           Funco Auxiliar
%--------------------------------------------------------------------------
function out=mycdf(x)
% Funo que retorna a probabilidade acumulada a partir de um vetor de
% densidade de probabilidade
global myx myy
xmax=x;
ymax=spline(myx,myy,xmax);
myxx=linspace(0,xmax,200);
myyy=spline(myx,myy,myxx);
out=trapz(myxx,myyy);
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%      DETERMINAO DA FUNO DENSIDADE MARGINAL  POSTERIORI DE p
%--------------------------------------------------------------------------
function out=f_pwitherror(x)
global m n r j t a1 b1 a2 b2 a b xint yint deltaxint
out=zeros(1,length(x));

% Inicializando variaveis necessrias
ajt=zeros(1,(r+1)*(n-r+1));
vp=zeros(1,(r+1)*(n-r+1));
vax=zeros(1,(r+1)*(n-r+1));
vbx=zeros(1,(r+1)*(n-r+1));

% Calculando as constantes
cont=0;
for j=0:r
    for t=0:(n-r)
        cont=cont+1;
        ax=a+n-j-t;     %alfa asterisco
        bx=b+j+t;       %beta asterisco
        ajt(cont)=nchoosek(r,j)*nchoosek(n-r,t)*beta(ax,bx)*f_k1*f_k2;
        vax(cont)=ax;
        vbx(cont)=bx;
    end
end
wjt=ajt/sum(ajt);

% Calculando o valor da funcao a posteriori para cada "x"
for i=1:length(x)
    out(1,i)=sum(wjt.*betapdf(x(i),vax,vbx));
end
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%                          INTEGRAL DE K1(j,t)
%--------------------------------------------------------------------------
function out=f_k1()
% funcao para obtencao da integral da funo k1(j,t) no intervalo e1=[0;1]
global m n r j t a1 b1 xint yint deltaxint
temp=quadl(@f_k1aux,1e-6,1-1e-6);
out=temp;



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%                          INTEGRAL DE K2(j,t)
%--------------------------------------------------------------------------
function out=f_k2()
% funcao para obtencao da integral da funo k2(j,t) no intervalo e1=[0;1]
global m n r j t a2 b2 xint yint deltaxint
temp=quadl(@f_k2aux,1e-6,1-1e-6);
out=temp;
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%                            FUNO K1(j,t)
%--------------------------------------------------------------------------
function out=f_k1aux(x)
% Calculo vetorial da funo k1 (j calcula em diversos pontos no intervalo
% [0,1], dado pelo vetor x). Repare que x = e1
global m n r j t a1 b1 xint yint deltaxint
indice=round(x/deltaxint)+1;
prob=yint(indice);
out=x.^(a1-1).*(1-x).^(b1-1).*(prob.^(r-j)).*((1-prob).^(n-r-t));
%--------------------------------------------------------------------------



%--------------------------------------------------------------------------
%                           Funco Auxiliar
%                            FUNO K2(j,t)
%--------------------------------------------------------------------------
function out=f_k2aux(x)
% Calculo vetorial da funo k2 (j calcula em diversos pontos no intervalo
% [0,1], dado pelo vetor x). Repare que x = e1
global m n r j t a2 b2 xint yint deltaxint
indice=round(x/deltaxint)+1;
prob=yint(indice);
out=x.^(a2-1).*(1-x).^(b2-1).*(prob.^(t)).*((1-prob).^(j));
%--------------------------------------------------------------------------
